/*===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 
 * IBM SDK, Java(tm) Technology Edition, v6
 * (C) Copyright IBM Corp. 2010, 2010. All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *===========================================================================
 */
/*
 * @(#)ParserDelegator.java	1.17 10/03/23
 *
 * Copyright (c) 2006, Oracle and/or its affiliates. All rights reserved.
 * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javax.swing.text.html.parser;

import javax.swing.text.html.HTMLEditorKit;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.ObjectInputStream;
import java.io.Reader;
import java.io.Serializable;
                                                                                //IBM-security_bulletin_tmp
import sun.awt.AppContext;                                                      //IBM-security_bulletin_tmp

/**
 * Responsible for starting up a new DocumentParser
 * each time its parse method is invoked. Stores a
 * reference to the dtd.
 *
 * @author  Sunita Mani
 * @version 1.17, 03/23/10
 */

public class ParserDelegator extends HTMLEditorKit.Parser implements Serializable {

    private static final Object DTD_KEY = new Object();                         //IBM-security_bulletin_tmp

    protected static synchronized void setDefaultDTD() {
    AppContext appContext = AppContext.getAppContext();                         //IBM-security_bulletin_tmp
    DTD dtd = (DTD) appContext.get(DTD_KEY);                                    //IBM-security_bulletin_tmp
    if (dtd == null) {                                                          //IBM-security_bulletin_tmp
	    DTD _dtd = null;
	    // (PENDING) Hate having to hard code!
	    String nm = "html32";
	    try {
		_dtd = DTD.getDTD(nm);
	    } catch (IOException e) {
		// (PENDING) UGLY!
		System.out.println("Throw an exception: could not get default dtd: " + nm);
	    }
	    dtd = createDTD(_dtd, nm);
		appContext.put(DTD_KEY, dtd);                                   //IBM-security_bulletin_tmp
        }
    }

    protected static DTD createDTD(DTD dtd, String name) {

	InputStream in = null;
	boolean debug = true;
	try {
	    String path = name + ".bdtd";
	    in = getResourceAsStream(path);
            if (in != null) {
                dtd.read(new DataInputStream(new BufferedInputStream(in)));
                dtd.putDTDHash(name, dtd);
	    }
        } catch (Exception e) {
            System.out.println(e);
        }
        return dtd;
    }


    public ParserDelegator() {
	  setDefaultDTD();                                                      //IBM-security_bulletin_tmp
	}

    public void parse(Reader r, HTMLEditorKit.ParserCallback cb, boolean ignoreCharSet) throws IOException {
	new DocumentParser((DTD) AppContext.getAppContext().get(DTD_KEY)).parse(r, cb, ignoreCharSet); //IBM-security_bulletin_tmp
    }

    /**
     * Fetch a resource relative to the ParserDelegator classfile.
     * If this is called on 1.2 the loading will occur under the
     * protection of a doPrivileged call to allow the ParserDelegator
     * to function when used in an applet.
     *
     * @param name the name of the resource, relative to the
     *  ParserDelegator class.
     * @returns a stream representing the resource
     */
    static InputStream getResourceAsStream(String name) {
	try {
            return ResourceLoader.getResourceAsStream(name);
	} catch (Throwable e) {
	    // If the class doesn't exist or we have some other 
	    // problem we just try to call getResourceAsStream directly.
	    return ParserDelegator.class.getResourceAsStream(name);
	}
    }

    private void readObject(ObjectInputStream s)
	throws ClassNotFoundException, IOException {
	 s.defaultReadObject();                                                 //IBM-security_bulletin_tmp
	 setDefaultDTD();                                                       //IBM-security_bulletin_tmp
	}
}


//IBM-security_bulletin_tmp
